##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

'''$Id: db.py,v 1.8 1999/11/23 15:41:31 petrilli Exp $'''
__version__='$Revision: 1.8 $'[11:-2]

import DCsapdb
from DateTime import DateTime
from Shared.DC.ZRDB.TM import TM

import string, sys
from string import strip, split, find
from time import time
from types import ListType

failures=0
calls=0
last_call_time=time()

class DB(TM):

    _p_oid=_p_changed=_registered=None

    Database_Connection=DCsapdb.connect
    Database_Error=DCsapdb.Error


    def __init__(self,connection):
        self.connection=connection
        self.kwargs = kwargs = self._parse_connection_string(connection)
	self.db=apply(self.Database_Connection, (), kwargs)
	self.cursor=self.db.cursor()
		
        
    def _parse_connection_string(self, connection):
        kwargs = {}
        items = split(connection)
        if not items: return kwargs
        db_host, items = items[0], items[1:]
        if '@' in db_host:
            db, host = split(db_host,'@',1)
            kwargs['database'] = db
            kwargs['host'] = host
        else:
            kwargs['database'] = db_host
        if not items: return kwargs
        kwargs['user'], items = items[0], items[1:]
        if not items: return kwargs
        kwargs['password'], items = items[0], items[1:]
        return kwargs


    def str(self,v, StringType=type('')):
        if v is None: return ''
        r=str(v)
        if r[-1:]=='L' and type(v) is not StringType: r=r[:-1]
        return r

    def _finish(self, *ignored):
        self.db.commit()

    def _abort(self, *ignored):
        self.db.rollback()

    def tables(self, rdb=0, _care=('TABLE', 'VIEW')):
#        if all:
#            t='ALL'
#            e=', OWNER'
#        else:
#            t='USER'
#            e=''
#        c=self._c
#        c.execute(
#            'select OBJECT_NAME, OBJECT_TYPE%s from %s_OBJECTS' % (e,t)
#            )
#        return c.fetchall()
#        c=self.cursor
#        try: r=c.execute('select * from %s' % table_name)
#        except: return ()
#        r=[]
#        a=r.append
#        for name, typ in self.db.objects():
#            if typ in _care:
#                a({'TABLE_NAME': name, 'TABLE_TYPE': typ})
#        return r
	pass

    def columns(self, table_name):
        c=self.cursor
        try: r=c.execute('select * from %s' % table_name)
        except: return ()
        desc=c.description
        r=[]
        a=r.append
        for name, type, width, scale in desc:
            a({ 'Name': name,
                'Type': type,
                'Precision': width,
                'Scale': scale,
                'Nullable': 0,
                })

        return r

    def query(self,query_string, max_rows=9999999):
        global failures, calls, last_call_time
        calls=calls+1
        desc=None
        result=()
        self._register()
        c=self.cursor            
        try:
            for qs in filter(None, map(strip,split(query_string, '\0'))):
                try:
			r=c.execute(qs)
		except self.Database_Error, mess:
			print mess.errorCode,mess.message
			if mess.errorCode == 100:
				r = None
			else: raise self.Database_Error(mess)
                if r is not None:
                    if desc is not None:
                        if c.description != desc:
                            raise 'Query Error', (
                                'Multiple select schema are not allowed'
                                )
                        if type(result) is not ListType: result=list(result)
                        if max_rows:
                            for row in c.fetchmany(max_rows-len(result)):
                                result.append(row)
                    else:
                        desc=c.description
                        if max_rows:
                            if max_rows==1: result=(c.fetchone(),)
                            else: result=c.fetchmany(max_rows)
                                
            failures=0
            last_call_time=time()
        except self.Database_Error, mess:
            failures=failures+1
            if (failures > 1000 or time()-last_call_time > 600):
                # Hm. maybe the db is hosed.  Let's try once to restart it.
                failures=0
                last_call_time=time()
		c.close()
		self.db.close()
		db=self.db=self.Database_Connection(self.connection)
		self.cursor=db.cursor()
                return self.query(query_string, max_rows)
	    else: raise sys.exc_type, sys.exc_value, sys.exc_traceback

        if desc is None: return (),()

        items=[]
        for i in range(len(desc)):
		name, typ, width, scale = desc[i]
		if typ in ['Integer','Smallint']: typ='i'
		elif typ in ['Fixed','Float']: typ='n'
		elif typ in ['Date','Time','Timestamp']: 
		    typ='d'
		    for j in range(len(result)):
                    	row = list(result[j])
                    	if type( row[i] ) == type(""):
                        	row[i] = DateTime( row[i] )
                        	result[j] = tuple(row)
		else: typ='s'
		items.append({
			'name': name,
			'type': typ,
			'width': width,
			'null': 0,
			})
        return items, result


